#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 Normal;
    vec2 texture_coord;
} vert;

/* Components of vectors are accessed by array indexing with the []-operator (indexing starts with 0) 
   or with the .-operator and the element names x, y, z, w or r, g, b, a or s, t, p, q:
*/

// view position
uniform vec3 viewPos;

// Material
uniform vec3 diffuseColor_in;
uniform vec4 illumination;

#define DIRLIGHTS_CNT 1
#define POINTLIGHTS_CNT 1

// directional light
struct directionalLightData {
    vec3 Direction;
    vec3 Color;
};
uniform directionalLightData directionalLights[DIRLIGHTS_CNT];

// point light
struct pointLightData { 
    vec3 Pos;
    vec3 Color;
    vec3 Attenuation;
};
uniform pointLightData pointLights[POINTLIGHTS_CNT];

uniform sampler2D myTexture;

out vec4 color;

void main() {
    vec3 norm = normalize(vert.Normal);
    vec3 viewDir = normalize(viewPos - vert.position_world);

    vec3 result = vec3(0.0); 

    vec4 FragColor = texture(myTexture, vert.texture_coord);
    vec3 diffuseColor = vec3(FragColor.r, FragColor.g, FragColor.b);

    // TEST ONLY Red/green to check u/v coordinates
    // vec3 diffuseColor = vec3(vert.texture_coord.x, vert.texture_coord.y, 0.0f);

    // directional lights
    for (int l=0; l<DIRLIGHTS_CNT; l++ )
    {
        // ambient
        // TODO: really add ambient per light?
        vec3 ambientDir = diffuseColor*illumination[0]; // ambient

        // diffuse 
        vec3 lightDir = normalize(-directionalLights[l].Direction);
	    float diff = max(dot(norm, lightDir), .0);
        vec3 diffuseDir = diff * directionalLights[l].Color *  diffuseColor*illumination[1]; // diffuse

        // specular
        vec3 reflectDir = reflect(-lightDir, norm);  
        float spec = pow(max(dot(viewDir, reflectDir), 0.0), illumination[3]); // shininess
        vec3 specularDir = directionalLights[l].Color * (spec * vec3(illumination[2]));  // specular

        result += ambientDir + diffuseDir + specularDir;
    }

    // point lights
    for (int l=0; l<POINTLIGHTS_CNT; l++ )
    {
        if ( (pointLights[l].Attenuation[0] == 0) &&
             (pointLights[l].Attenuation[1] == 0) &&
             (pointLights[l].Attenuation[2] == 0) )
             break;

        // ambient
        vec3 ambientPoint = diffuseColor*illumination[0]; // ambient

        // diffuse 
	    vec3 lightDir = normalize(pointLights[l].Pos - vert.position_world);
	    float diff = max(dot(norm, lightDir), .0);
        vec3 diffusePoint = diff * pointLights[l].Color *  diffuseColor*illumination[1]; // diffuse

        // specular
        vec3 reflectDir = reflect(-lightDir, norm);  
        float spec = pow(max(dot(viewDir, reflectDir), 0.0), illumination[3]); // shininess
        vec3 specularPoint = pointLights[l].Color * (spec * vec3(illumination[2]));  // specular

        // attenuation
        float distance    = length(pointLights[l].Pos - vert.position_world);
        float attenuation = 1.0 / (pointLights[l].Attenuation[0] + pointLights[l].Attenuation[1] * distance 
                                    + pointLights[l].Attenuation[2] * (distance * distance));  

        result += (ambientPoint + diffusePoint + specularPoint)*attenuation;
    }

	color = vec4(result, 1.0);

}